

import java.lang.*;
import java.awt.*;
import java.applet.*;
//import java.awt.event.*;
import java.io.*;
import java.net.*;


public class OutFrame extends Frame {
  double m_dSimLen;
  double m_dDelta;
  
  
  /* Values received from itterator */
  public double m_dThroughput[];
  public double m_dN[];
  
  /* Max / min values */
  public double m_dMaxN;
  public double m_dMinN;

  public double m_dMaxTh;
  public double m_dMinTh;

  /* What to draw */
  public double m_dMinDraw;
  public double m_dMaxDraw;
  public double m_dDataDraw[];

  public String m_strNameDraw;

  public boolean m_bDrawN;

  public int m_nPoints;
  public int m_nCrtIndex;

  OutFrame( double dSimLen, double dDelta, 
	    Socket socket  ) throws Exception {
    super( "Output Window" );
    
    m_dSimLen = dSimLen;
    m_dDelta = dDelta;

    m_nPoints = ( int ) ( dSimLen / dDelta ) - 1;
    m_dThroughput = new double[ m_nPoints ];
    m_dN = new double[ m_nPoints ];
    
    m_dMaxN = m_dMaxTh = 0;
    m_dMinN = m_dMinTh = 10000000.00;
    m_nCrtIndex = 0;

    m_bDrawN = true;
    
    m_strNameDraw = new String( "N( t )" );
    /* Put the menu there */
    buildMenuBar();

    /* Set a croshair cursor for this */
    setCursor( CROSSHAIR_CURSOR );
  }


  /* the painting one .. */
  public final static int INIT_XDIM = 700;
  public final static int INIT_YDIM = 500;


  public final static int LEFT_MARGIN = 50;
  public final static int BOTTOM_MARGIN = 20;
  public final static int TOP_MARGIN = 20;
  public final static int RIGHT_MARGIN = 30;

  public final static int VERTICAL_DIVISIONS = 12;
  public final static int HORIZONTAL_DIVISIONS = 12;
  
  public void paint( Graphics g ) {
    /* first get the size of the window */ 
    int cx = size().width;
    int cy = size().height;

    /* Erase all first */
    g.setColor( Color.white );
    g.fillRect( 0, 0, cx, cy );

    int leftMargin = LEFT_MARGIN + insets().left;
    int rightMargin = RIGHT_MARGIN + insets().right;
    int topMargin = TOP_MARGIN + insets().top;
    int bottomMargin = BOTTOM_MARGIN + insets().left;


    /* draw the axes */
    g.setColor( Color.blue );
    g.drawLine( leftMargin - LEFT_MARGIN / 2, cy - bottomMargin, 
		cx - rightMargin + RIGHT_MARGIN / 2, cy - bottomMargin );
    g.drawLine( leftMargin, topMargin - TOP_MARGIN / 2, 
		leftMargin, cy - bottomMargin + BOTTOM_MARGIN / 2 );

    /* draw the hors arrows */
    g.drawLine( cx - rightMargin + RIGHT_MARGIN / 2, cy - bottomMargin,
		cx - rightMargin + RIGHT_MARGIN / 2 - 7, cy - bottomMargin - 2 ) ;
    g.drawLine( cx - rightMargin + RIGHT_MARGIN / 2, cy - bottomMargin, 
		cx - rightMargin + RIGHT_MARGIN / 2 - 7, cy - bottomMargin + 2 ) ;
    
    /* draw the vertical arrows */
    g.drawLine( leftMargin, topMargin - TOP_MARGIN/2,
		leftMargin - 2, topMargin - TOP_MARGIN/2 + 7 );
    g.drawLine( leftMargin, topMargin - TOP_MARGIN/2,
		leftMargin + 2, topMargin - TOP_MARGIN/2 + 7 );

      
    /* shrink the rectangle */
    cx = cx - leftMargin - rightMargin;
    cy = cy - topMargin - bottomMargin;
    
    /* Now, compute the scaling factors */
     int nLast;
    if( m_nCrtIndex >= m_nPoints )
      nLast = m_nPoints - 1;
    else
      nLast = m_nCrtIndex;

    double dFactorX = ( ( double ) cx ) / (( double )( nLast + 1 ));
    double dFactorY = ( cy / ( m_dMaxDraw - m_dMinDraw ));
    
    /* draw the scales */
    double dHorzDivision = (( double )( nLast + 1 ))/ (( double )HORIZONTAL_DIVISIONS );
    double dVertDivision = ( m_dMaxDraw - m_dMinDraw ) / VERTICAL_DIVISIONS;

    /* vertical */
    for( int nIndex = 0; nIndex < VERTICAL_DIVISIONS; nIndex ++ ) {
      g.setColor( Color.blue );
      g.drawLine( leftMargin ,
		  ( int )( cy + topMargin - ((dVertDivision * ( nIndex + 1 )) * dFactorY )),
		  leftMargin - 3,
		  ( int )( cy + topMargin - ((dVertDivision * ( nIndex + 1 )) * dFactorY )) );     
      g.setColor( Color.lightGray );
      g.drawLine( leftMargin,
		  ( int )( cy + topMargin - ((dVertDivision * ( nIndex + 1 )) * dFactorY )),
		  leftMargin + cx ,
		  ( int )( cy + topMargin - ((dVertDivision * ( nIndex + 1 )) * dFactorY )) );  

      /* Put the label */
      Double d = new Double( dVertDivision * ( nIndex + 1 ) + m_dMinDraw );
      g.setColor( Color.blue );
      String putDouble = d.toString();
      int toDelete = putDouble.length() - putDouble.indexOf( "." ) - 3;
      if( toDelete < 0 )
	toDelete = 0;
      putDouble = putDouble.substring( 0, putDouble.length() - toDelete );
      
      g.drawString( putDouble,
		    leftMargin - 5 - g.getFontMetrics().stringWidth( putDouble.toString () ),
		    ( int )( cy + topMargin - ((dVertDivision * ( nIndex + 1 )) * dFactorY )));
			     
      
    }

    /* horizontal */
    for( int nIndex = 0; nIndex < HORIZONTAL_DIVISIONS; nIndex ++ ) {
      g.setColor( Color.blue );
      g.drawLine( ( int ) ( ( nIndex + 1 ) * dHorzDivision * dFactorX + leftMargin ),
		  cy + topMargin, 
		  ( int ) ( ( nIndex + 1 ) * dHorzDivision * dFactorX + leftMargin ),
		  cy + topMargin + 3 );
      g.setColor( Color.lightGray );
      g.drawLine( ( int ) ( ( nIndex + 1 ) * dHorzDivision * dFactorX + leftMargin ),
		  topMargin, 
		  ( int ) ( ( nIndex + 1 ) * dHorzDivision * dFactorX + leftMargin ),
		  cy + topMargin );

      /* Put the label */
      Double d = new Double( ( nIndex + 1 ) * dHorzDivision * m_dDelta );
      g.setColor( Color.blue );
      String putDouble = d.toString();
      int toDelete = putDouble.length() - putDouble.indexOf( "." ) - 3;
      if( toDelete < 0 )
	toDelete = 0;
      putDouble = putDouble.substring( 0, putDouble.length() - toDelete );

      g.drawString( putDouble,
		    ( int ) ( ( nIndex + 1 ) * dHorzDivision * dFactorX + leftMargin - 
			       g.getFontMetrics().stringWidth( putDouble.toString () ) / 2 ),
		    cy + topMargin + 5 + g.getFontMetrics().getAscent() );
    }

    /* write the text for the axis */
    String timeAxis = new String("Time");
    g.setColor( Color.blue );
    g.drawString( timeAxis, cx + leftMargin -
		  g.getFontMetrics().stringWidth( timeAxis ), 
		  cy + topMargin - 4 );

    g.drawString( m_strNameDraw, leftMargin + 4,
		  topMargin - TOP_MARGIN/2 - 4 + g.getFontMetrics().getAscent() );
   
    /* The values .. */
    g.setColor( Color.red );
    for( int nIndex = 0; nIndex < nLast-1; nIndex ++ ) {
      g.drawLine( ( int ) ( ( nIndex + 1 ) * dFactorX + leftMargin ),
		( int )( cy + topMargin - ((m_dDataDraw[ nIndex ] - m_dMinDraw) * dFactorY )),
		( int )((nIndex + 2) * dFactorX + leftMargin),
		( int )( cy + topMargin - ((m_dDataDraw[ nIndex +1 ] - m_dMinDraw) * dFactorY )));
      if( m_itemOvals.getState() )
	g.fillOval( ( int ) ( ( nIndex + 1 ) * dFactorX + leftMargin ) - 2,
		    ( int )( cy + topMargin - ((m_dDataDraw[ nIndex ] - m_dMinDraw) * dFactorY )) - 2,
		    5, 5 );
    }
  }
  
  /* Some menu items .. */
  MenuItem m_itemClose;
  MenuItem m_itemSave;
  MenuItem m_itemDrawN;
  MenuItem m_itemDrawTh;
  CheckboxMenuItem m_itemOvals;

  /* this builds up the menu bar */
  void buildMenuBar() {
    
    /* Create the menu items .. */
    m_itemClose = new MenuItem( "Close Window" );
    m_itemSave = new MenuItem( "Save Data..." );
    m_itemDrawN = new MenuItem( "Graph N" );
    m_itemDrawN.enable( ! m_bDrawN );
    m_itemDrawTh = new MenuItem( "Graph Throughput" );
    m_itemDrawN.enable( m_bDrawN );
    m_itemOvals = new CheckboxMenuItem( "Point the Values" );
    m_itemOvals.setState( false );

    /* Cook the options menu */
    Menu optionsMenu = new Menu("Options");

    optionsMenu.add( m_itemDrawN );
    optionsMenu.add( m_itemDrawTh );
    optionsMenu.addSeparator();
    optionsMenu.add( m_itemOvals );
    
    /* Cook the file Menu */
    Menu fileMenu = new Menu("File");

    fileMenu.add( m_itemSave );
    fileMenu.addSeparator();
    fileMenu.add( m_itemClose );
    
    /* Cook the menu bar */
    MenuBar mb = new MenuBar();

    mb.add( fileMenu );
    mb.add( optionsMenu );
    
    setMenuBar( mb );
  }

  /* Now.. the actions */
  public boolean action( Event e, Object w ) {
    if( e.target == m_itemClose ){
      dispose();
    }
    else 
      if( e.target == m_itemSave ) {
	/* To be implemented soon */
      }
    else
      if( e.target == m_itemDrawN ) {
	m_dMinDraw = m_dMinN;
	m_dMaxDraw = m_dMaxN;
	m_dDataDraw = m_dN;

	m_strNameDraw = new String( "N( t )" );

	m_bDrawN = true;

	m_itemDrawN.enable( false );
	m_itemDrawTh.enable( true );

	repaint();
      }

    else
      if( e.target == m_itemDrawTh ) {
	m_dMinDraw = m_dMinTh;
	m_dMaxDraw = m_dMaxTh;
	m_dDataDraw = m_dThroughput;

	m_strNameDraw = new String( "Th( t )" );

	m_bDrawN = false;

	m_itemDrawN.enable( true );
	m_itemDrawTh.enable( false );

	repaint();
      }
    else
      if( e.target == m_itemOvals )
	repaint();

    return false;
  }

  public void update(Graphics g) {
    paint( g );
  }
}

