/*
 * Applet : Z-Itteration applet
 *
 * An interface for the server which computes
 * some parameters for a sefl configurating network using
 * Z-itteration K. implementation for a 
 */


import java.lang.*;
import java.awt.*;
import java.applet.*;
//import java.awt.event.*;
import java.io.*;
import java.net.*;

import ReaderThread;
import OutFrame;
import NetCanvas;

public class ZIterator extends Applet /* implements ActionListener */{

  /* The network drawing object( canvas ) */
  NetCanvas m_theCanvas;

  /* The Button to ask the server for processing */
  Button m_theEvalButton;

  /* Save / load buttons */
  Button m_theSaveButton;
  Button m_theLoadButton;

  /* The Label for messages */
  Label m_theMsgLabel;
  
  /* Network stuff .. */
  String m_strServerAddress;
  Integer m_nPort;
  
  /* Z- itteretion parameters */
  Double m_dSimLen;
  Double m_dDelta;
  Double m_dRecompute;

  int m_nSteps;

  int m_nRouting;
  /* Some limitations for these parameters */
  public final static double MIN_SIMLEN  = 1;
  public final static double MAX_SIMLEN  = 80;
  
  public final static double MIN_DELTA = 0.02;
  public final static double MAX_DELTA = 0.2;

  public final static double MIN_EPSILON = 0.00001;
  public final static double MAX_EPSILON = 0.001;

  public final static int MAX_STEPS = 400;
  
  public final static double MIN_RECOMPUTE = 1;

  /* Some inputs - double for the parameters above */
  
  Choice m_choiceRouting;
  TextField m_textSimLen;
  TextField m_textDelta;
  TextField m_textRecompute;

  public void init() {

    /* Get the address of the server */
    m_strServerAddress = getParameter( "server" );

    String strPort = getParameter( "port" );
    if( strPort == null )
      m_nPort = new Integer( 15000 );
    else
      m_nPort = new Integer( strPort );
    
    /* init the Z parameters */
    m_dSimLen = new Double( 20.0 );
    m_dDelta = new Double( 0.1 );
    m_dRecompute = new Double( 10.0 );
    
    /* set the layout */
    setLayout(new BorderLayout( 5, 5 ));

    /* Now creates the components */

    m_theMsgLabel = new Label("                                                                        ");
    add( "North", m_theMsgLabel );

    m_theCanvas = new NetCanvas( m_theMsgLabel );
    add( "Center", m_theCanvas );
    
    /* the rigt panel .. */
    Panel leftPanel = new Panel();
    //    leftPanel.setBackground( Color.white );
    leftPanel.setLayout( new GridLayout( 11, 1, 5, 5 ));

    /*  Add some inputs to get parameters */
    
    m_choiceRouting = new Choice();
    m_choiceRouting.addItem( " Hop" );
    m_choiceRouting.addItem( " Util " );
    m_choiceRouting.addItem( " Hop - Util " );

    leftPanel.add( new Label( "Weights:" ));
    leftPanel.add( m_choiceRouting );

    /* Prepare the text inputs for Z - itteration parameters */
    m_textSimLen = new TextField( m_dSimLen.toString(), 10 );
    m_textDelta = new TextField( m_dDelta.toString(), 10 );
    m_textRecompute = new TextField( m_dRecompute.toString(), 10 );

    /* add them to the right panel */
    leftPanel.add( new Label( "Eval. Time:" ));
    leftPanel.add( m_textSimLen );
    leftPanel.add( new Label( "Delta:" ));
    leftPanel.add( m_textDelta );
    leftPanel.add( new Label( "Recompute Weights:" ));
    leftPanel.add( m_textRecompute );

    /* prpare the "Load / Save " button */
    m_theLoadButton = new Button("Load...");
    leftPanel.add( m_theLoadButton );
    //    m_theLoadButton.addActionListener( this );

    m_theSaveButton = new Button("Save...");
    leftPanel.add( m_theSaveButton );
    //    m_theSaveButton.addActionListener( this );

    /* prpare the "Evaluate" button */
    m_theEvalButton = new Button("Evaluate");
    leftPanel.add( m_theEvalButton );
    //    m_theEvalButton.addActionListener( this );
    
    add( "East", leftPanel );
 
  } 
  
  // public void actionPerformed(ActionEvent e) {
  public boolean action( Event e, Object what ) {
    //    if( e.getSource() == m_theEvalButton ) {
    if( e.target == m_theEvalButton ) {
      /* Do the server interaction */
      doDialog();
    }
    else
      //  if( e.getSource() == m_theLoadButton ) {
      if( e.target == m_theLoadButton ) {
      /* Do the server interaction */
      doLoad();
    }
    else
      //if( e.getSource() == m_theSaveButton ) {
      if( e.target == m_theSaveButton ) {
     /* Do the server interaction */
      doSave();
    }
    
    //
    return false;
  }
  

  public boolean readParameters() {
    /* Get the parameters from the input */
    m_nRouting = m_choiceRouting.getSelectedIndex() + 1;
    
    /* the other parameters */
    try {
      m_dSimLen = new Double( m_textSimLen.getText( ));
      m_dDelta = new Double( m_textDelta.getText( ));
      m_dRecompute = new Double( m_textRecompute.getText( ));
    }
    catch( Exception e ) {
      m_theMsgLabel.setText( "Error in the input !!" );
      return false;
    }

    /* check for bounds */
    boolean bOutOfBounds = false;
    if( m_dDelta.doubleValue() > MAX_DELTA ) {
      m_dDelta = new Double( MAX_DELTA );
      bOutOfBounds = true; 
    }
    if( m_dDelta.doubleValue() < MIN_DELTA ) {
      m_dDelta = new Double( MIN_DELTA );
      bOutOfBounds = true;
    }

    if( m_dSimLen.doubleValue() < MIN_SIMLEN ) {
      m_dSimLen = new Double( MIN_SIMLEN );
      bOutOfBounds = true;
    }
    if( m_dSimLen.doubleValue() > MAX_SIMLEN ) {
      m_dSimLen = new Double( MAX_SIMLEN );
      bOutOfBounds = true;
    }

    if( m_dRecompute.intValue() < MIN_RECOMPUTE ) {
      m_dRecompute = new Double( MIN_RECOMPUTE );
      bOutOfBounds = true;
    }

    
    /* print a message to the user */
    if( bOutOfBounds ) {
      m_theMsgLabel.setText("Parameters out of range. Were modified. Try again." );
      m_textSimLen.setText( m_dSimLen.toString() );
      m_textDelta.setText( m_dDelta.toString() );
      m_textRecompute.setText( m_dRecompute.toString() );
    }
    
    return !bOutOfBounds;
  }

  public void doDialog() {
    if (! readParameters() ) 
      return;
    
    try {
      Socket s = new Socket( m_strServerAddress, m_nPort.intValue() );
      /* Message to user */
      m_theMsgLabel.setText( "Connecting to the server .. " );

     
      /* Get the in / out streams */
      DataOutputStream os = new DataOutputStream
	( s.getOutputStream() );

      /* Message to user */
      m_theMsgLabel.setText( "Sending Data.. " );

      /* Send general parameters */
      String sendStr;
      sendStr = m_dDelta.doubleValue() + " " +
	m_dSimLen.doubleValue() + " 0.0001 " + 
	m_dRecompute.doubleValue() + " ";
      
      //
      byte buffer[] = new byte[ sendStr.length() ];
      sendStr.getBytes( 0, sendStr.length(), buffer, 0 ); 
      os.write( buffer );

      /* Push in the numbers */
      m_theCanvas.OutputNetConfig( os, m_nRouting );

      os.flush();

      /* Message to user */
      m_theMsgLabel.setText( "Data Sent OK... " );

      /* create the window for output */ 
      OutFrame ow = new OutFrame( 
				 m_dSimLen.doubleValue(), 
				 m_dDelta.doubleValue(), 
				 s );
      
      ow.setTitle(" Results " );
      ow.pack();
      ow.resize( OutFrame.INIT_XDIM, OutFrame.INIT_YDIM );
      ow.show();

      ReaderThread rt = new ReaderThread( s, ow );
      rt.start();

    }

    catch( Exception e ) {
      /* Message to user */
      m_theMsgLabel.setText( "Error in network connection .. " );
    }
  
  }
  
  /* this will save the current net to a file */
  void doSave() {
    /* find a parent frame */
    Object findFrame = getParent();
    while( !(findFrame instanceof Frame ) )
      findFrame = (( Component )findFrame).getParent();
    String fileName = new String( "outnet" );

    try {
      /* display the file dialog */
      FileDialog fd = new FileDialog( ( Frame ) findFrame, "Save Network", FileDialog.SAVE );
      fd.show( );
      /* save to that file */
      fileName = fd.getFile( );
    }
    catch( AWTError  e ) {
      m_theMsgLabel.setText( "Unimplemented File dialog - use the ugly one " );
      SimpleDialog sd = new SimpleDialog( ( Frame ) findFrame, "Enter the file name :", 40 );
      sd.move/*setLocation*/( 100, 200 );
      sd.show();

      fileName = sd.output;
    }

    /* first get the other parameters */
    readParameters();

    try {
      FileOutputStream fw = new FileOutputStream( fileName );
      DataOutputStream output = new DataOutputStream( fw );

      output.writeDouble( m_dDelta.doubleValue() );
      output.writeDouble( m_dSimLen.doubleValue() );
      output.writeDouble( m_dRecompute.doubleValue() );
      output.writeInt( m_nRouting );

      /* this is pretty ugly.. but you know.. */
      m_theCanvas.OutputNetConfigToFile( output );
      
      fw.flush();
      fw.close();
    }
    catch ( Exception e ) {
      /* Message to user */
      m_theMsgLabel.setText( "Error while saving .. " );
    }
  }

  void doLoad() {
    /* find a parent frame */
    Object findFrame = getParent();
    while( !(findFrame instanceof Frame ) )
      findFrame = (( Component )findFrame ).getParent();

    String fileName = new String( "outnet" );

    try {
      /* display the file dialog */
      FileDialog fd = new FileDialog( ( Frame ) findFrame, "Save Network", FileDialog.LOAD );
      fd.show( );
      /* save to that file */
      fileName = fd.getFile( );
    }
    catch( AWTError e ) {
      m_theMsgLabel.setText( "Unimplemented File dialog - use the ugly one " );
      SimpleDialog sd = new SimpleDialog( ( Frame ) findFrame, "Enter the file name :", 40 );
      sd.move/*setLocation*/( 100, 200 );
      sd.show();

      fileName = sd.output;
    }


    /* Read the file */
    try {
      FileInputStream fr = new FileInputStream(fileName);
      DataInputStream input = new DataInputStream( fr );

      m_dDelta = new Double( input.readDouble( ));
      m_dSimLen = new Double( input.readDouble( ));
      m_dRecompute = new Double( input.readDouble( ));
      m_nRouting = input.readInt( );

      /* this is pretty ugly.. but you know.. */
      m_theCanvas.InputNetConfigFromFile( input );
      
      fr.close();
      
    }
    catch ( Exception e ) {
      /* set the old values .. */
      m_dSimLen = new Double( 20.0 );
      m_dDelta = new Double( 0.1 );
      m_dRecompute = new Double( 10.0 );
      m_nRouting = 1;

      /* Message to user */
      m_theMsgLabel.setText( "Error while reading .. " );
    }
    
    /* Update the applet */
    
    /* Update the data in controld */
    m_textSimLen.setText( m_dSimLen.toString() );
    m_textDelta.setText( m_dDelta.toString() );
    m_textRecompute.setText( m_dRecompute.toString() );
    
    m_choiceRouting.select( m_nRouting - 1 );
    m_theCanvas.repaint(); 
  }
}

  
